/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Bits.c	40.2      meg      10/26/89"               */

#include "LookupDefs.i"
extern FILE *SummaryFile;
extern FILE *TruthTableFile;
extern int TruthTableFlag;

AssertSignal(StateName, Signal, Level)
/*********************************************************************
 * Outputs are associated with states (Moore machine model).  For each
 * row vector associated with a state, assert the given signal.
 *********************************************************************
 */
  char *StateName, *Signal, Level;
{ int temp;
  struct State * StateFromName(), *LocatedState;
  extern char * ProgramName;
  struct StateVectorRow *p;

  temp=TypeFromName(StateName);
  if(temp!=STATE)
  {  fprintf(stderr,
     "%s(AssertSignal):  %s is not a State\n", ProgramName, StateName);
     exit(1);
  }
  LocatedState=StateFromName(StateName);
  if(LocatedState->vectors==NULL)
  {  fprintf(stderr,
     "%s(AssertSignal):  No vectors in State %s\n", ProgramName, StateName);
     exit(1);
  }
  p=LocatedState->vectors;
  while(p!=NULL)
  {  SetBit(p, Signal, Level);
     p=p->next;
  }
} /*AssertSignal*/


SetBit(Vector, Signal, Level)
/*******************************************************
 * Set the given signal to the given level in the vector
 *******************************************************
 */
   struct StateVectorRow *Vector;
   char *Signal;
   char Level;

{  int SignalType;
   static int index;
   extern char * ProgramName;
   extern int StateVectorSize, NumberOfOutputs;

   SignalType=TypeFromName(Signal);
   if((SignalType==ILLEGAL)||(SignalType==STATE))
   {  fprintf(stderr, "%s(SetBit):  Bad signal %s.\n", ProgramName, Signal);
      exit(1);
   }
   /*
   ** If the level is not an output and the level is not 0 or 1, panic
   */
   if((Level!=ONE)&&(Level!=ZERO)&&(index<StateVectorSize-NumberOfOutputs))
   { fprintf(stderr,
     "%s(SetBit):  Bad level %c for signal %s\n", ProgramName, Level, Signal);
     exit(1);
   }
   index=NumberFromName(Signal);
   if(Vector->bits[index]!=DONTCARE )
   {  if ( index >= StateVectorSize-NumberOfOutputs && 
   			Vector->bits[index]==ZERO )
      { /* Do Nothing, Outputs have default value of zero */ }
      else {
   	fprintf(stderr,
	      "%s(SetBit):  Resetting %s from %c to %c\n", ProgramName, Signal,
	      Vector->bits[index], Level);
      	exit(1);
      }
   }
   Vector->bits[index]=Level;
} /*SetBit*/


char
ReadBit(Vector, Signal)
/******************************************************************
 * Return the value associated with the given signal in the vector.
 ******************************************************************
 */
   struct StateVectorRow *Vector;
   char *Signal;
{  int NameType, SignalIndex;
   extern char * ProgramName;
   if(Vector==NULL)
   {  fprintf(stderr, "%s(ReadBit):  NULL vector\n", ProgramName);
      exit(1);
   }
   NameType=TypeFromName(Signal);
   if((NameType!=OUTPUTS)&&(NameType!=INPUTS))
   {  fprintf(stderr, 
      "%s(ReadBit):  Identifier %s is not a signal\n", ProgramName, Signal);
      exit(1);
   }
   SignalIndex=NumberFromName(Signal);
   return(Vector->bits[SignalIndex]);
} /*ReadBit*/


CopyOutputs(Into, From)
/*
 * Copy the output bits from FROM into INTO
 */
   struct StateVectorRow * Into, * From;
{  extern int NumberOfInputs, NumberOfOutputs, NumberOfStateBits;
   int loop, start;
   start=NumberOfInputs+2*NumberOfStateBits;
   for(loop=0;loop<NumberOfOutputs;loop++)
      Into->bits[loop+start]=From->bits[loop+start];
} /*CopyOutputs*/


