/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Print.c	40.3      meg      11/28/89"               */


#include "LookupDefs.i"
extern FILE *SummaryFile;
extern FILE *TruthTableFile, *ImplicantFile;
extern int TableFlag, ImplicantFlag, TruthTableFlag;
extern int NumberOfInputs, NumberOfStateBits;

PrintVector(CurrentVector, StateName)
/*
 * Print a single vector in a nice fashion.
 */
   struct StateVectorRow * CurrentVector;
   char 		 * StateName;
{  extern int StateVectorSize;
   extern char * ProgramName;
   int index;

  if(CurrentVector!=NULL)
  {  for(index=0; index<StateVectorSize;  index++) {
	   if(TableFlag)
	   	fprintf(SummaryFile, "  %c", CurrentVector->bits[index]);
	   if(TruthTableFlag)
	   	fprintf(TruthTableFile, "%c", CurrentVector->bits[index]);
     }
     if(TableFlag)
     	fprintf(SummaryFile, "   %s\n   ", StateName); 
     if(TruthTableFlag)
   	fprintf(TruthTableFile, "\n");
     if(ImplicantFlag) {
    	for(index=0; index<NumberOfInputs;  index++) 
		fprintf(ImplicantFile, "%c", CurrentVector->bits[index]);
	fprintf(ImplicantFile, "  %-18s   %-18s  ", 
				StateName, CurrentVector->StateName);
    	for(index=NumberOfInputs+2*NumberOfStateBits; index<StateVectorSize; index++) 
		fprintf(ImplicantFile, "%c", CurrentVector->bits[index]);
	fprintf(ImplicantFile, " \n");
     }
  }
  else
  {  fprintf(stderr, "%s(PrintVector):  null vector\n", ProgramName);
     exit(1);
  }
} /*PrintVector*/


PrintVectors(List, StateName)
/*
 * Print a list of vectors in a nicely formatted fashion
 */
   struct StateVectorRow * List;
   char 		 * StateName;
{  struct StateVectorRow *temp;

   temp=List;
   while(temp!=NULL)
   {  PrintVector(temp, StateName);
      temp=temp->next;
   }
   if (TableFlag)
   	fprintf(SummaryFile, "\n   ");

} /*PrintVectors*/


PrintTable()
/************************************
 * Print the state table for the fsm.
 ************************************
 */
{ extern struct State *StateList;
  struct State *CurrentState;

  PrintInputNames();
  PrintOutputNames();
  PrintHeading();
  CurrentState=StateList;
  while(CurrentState!=NULL)
  {  PrintVectors(CurrentState->vectors, CurrentState->name);
     CurrentState=CurrentState->next;
  }
  if (TableFlag)
   	fprintf(SummaryFile, "\n");
  if (TruthTableFlag)
	fprintf(TruthTableFile, ".e\n");
} /*PrintTable*/


PrintHeading()
/*****************************************
 * Print a heading across the state table.
 *****************************************
 */
{ int i;
  extern int StateVectorSize, NumberOfStateBits, NumberOfInputs,
	     NumberOfOutputs;
  if(TruthTableFlag)
	fprintf(TruthTableFile, ".p -1\n");
  if(TableFlag) {
  	fprintf(SummaryFile, "\n\nState Table\n   ");
  	for(i=0; i<NumberOfInputs   ; i++) fprintf(SummaryFile, "  i");
  	for(i=0; i<NumberOfStateBits; i++) fprintf(SummaryFile, "  s");
  	for(i=0; i<NumberOfStateBits; i++) fprintf(SummaryFile, "  n");
  	for(i=0; i<NumberOfOutputs  ; i++) fprintf(SummaryFile, "  o");
  	fprintf(SummaryFile, "\n   ");
	
  	for(i=0; i<NumberOfInputs   ; i++) fprintf(SummaryFile,"%3d",   i);
  	for(i=0; i<NumberOfStateBits; i++) fprintf(SummaryFile,"%3d",   i);
  	for(i=NumberOfStateBits; i>0; i--) fprintf(SummaryFile,"%3d", i-1);
  	for(i=0; i<NumberOfOutputs  ; i++) fprintf(SummaryFile,"%3d",   i);
  	fprintf(SummaryFile, "\n   ");
  	for(i=0; i<StateVectorSize  ; i++) fprintf(SummaryFile,"---");
  	fprintf(SummaryFile, "\n   ");
   }
} /*PrintHeading*/


PrintInputNames()
/**********************************************************************
 * Print the names of input variables and the numbers assigned to them.
 **********************************************************************
 */
{ extern int NumberOfInputs, NumberOfStateBits;
  extern char * ProgramName;
  int i;
  char *name, *NameFromTypeNumber();
  if(TableFlag)
  	fprintf(SummaryFile, "\n\nINPUTS:\n");
  if(TruthTableFlag)
	fprintf(TruthTableFile, ".ilb ");
  if(ImplicantFlag)
	fprintf(ImplicantFile, ".ilb ");
  for(i=0; i<NumberOfInputs; i++)
  {  name=NameFromTypeNumber(INPUTS, i);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintInputNames):  Input %d not found\n", ProgramName, i);
       exit(1);
     }
     if(TableFlag)
     	fprintf(SummaryFile, "   i%02d:  %s\n", i, name);
     if(TruthTableFlag)
	fprintf(TruthTableFile, "%s ", name);
     if(ImplicantFlag)
	fprintf(ImplicantFile, "%s ", name);

  }

  for(i=0; i<NumberOfStateBits; i++)
  {  name=NameFromTypeNumber(INPUTS, NumberOfInputs+i);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintInputNames):  Input %d not found\n", ProgramName, i);
       exit(1);
     }
     if(TableFlag)
     	fprintf(SummaryFile, "   s%02d:  %s", i, name);
     if(TruthTableFlag)
	fprintf(TruthTableFile, "%s ", name);
     if(ImplicantFlag)
	fprintf(ImplicantFile, "%s ", name);
     if(i==0 && TableFlag)
	fprintf(SummaryFile, " (msb)\n");
     else if(i==NumberOfStateBits-1 && TableFlag)
	fprintf(SummaryFile, " (lsb)\n");
     else if(TableFlag)
     	fprintf(SummaryFile, "\n");
  }
  if(TruthTableFlag)
	fprintf(TruthTableFile, "\n");
  if(ImplicantFlag)
	fprintf(ImplicantFile, "\n");
} /*PrintInputNames*/


PrintOutputNames()
/***********************************************************************
 * Print the names of output variables and the numbers assigned to them.
 ***********************************************************************
 */
{ extern int NumberOfOutputs, NumberOfStateBits, NumberOfInputs;
  int i;
  char *name, *NameFromTypeNumber();
  extern char * ProgramName;
  if(TableFlag)
  	fprintf(SummaryFile, "\n\nOUTPUTS:\n");
  if(TruthTableFlag)
	fprintf(TruthTableFile, ".ob ");
  if(ImplicantFlag)
	fprintf(ImplicantFile, ".ob ");
  for(i=0; i<NumberOfStateBits; i++)
  {  name=NameFromTypeNumber(OUTPUTS, i+NumberOfInputs+NumberOfStateBits);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintOutputNames):  Output %d not found\n", ProgramName, i);
       exit(1);
     }
     if(TableFlag)
     	fprintf(SummaryFile, "   n%02d:  %s", NumberOfStateBits-i-1, name);
     if(TruthTableFlag)
	fprintf(TruthTableFile, "%s ", name);
     if(ImplicantFlag)
	fprintf(ImplicantFile, "%s ", name);
     if(i==0 && TableFlag)
	fprintf(SummaryFile, " (lsb)\n");
     else if(i==NumberOfStateBits-1 && TableFlag)
	fprintf(SummaryFile, " (msb)\n");
     else if(TableFlag)
	fprintf(SummaryFile, "\n");
  }
  for(i=0; i<NumberOfOutputs; i++)
  {  name=NameFromTypeNumber(OUTPUTS, i+NumberOfInputs+2*NumberOfStateBits);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintOutputNames):  Output %d not found\n", ProgramName, i);
       exit(1);
     }
     if(TableFlag)
     	fprintf(SummaryFile, "   o%02d:  %s\n", i, name);
     if(TruthTableFlag)
	fprintf(TruthTableFile, "%s ", name);
     if(ImplicantFlag)
	fprintf(ImplicantFile, "%s ", name);
  }
  if(TruthTableFlag)
	fprintf(TruthTableFile, "\n");
  if(ImplicantFlag)
	fprintf(ImplicantFile, "\n");
} /*PrintOutputNames*/

