/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Slang.c	40.1      meg      6/29/88"               */

#include "LookupDefs.i"

PrintSlang()
/********************************************************************
 * For each output create a node that descibes its function.
 */

{ int index, Dindex, AlreadyFoundOne, Eliminated;
  char *name, ReadBit(), *NameFromTypeNumber();
  extern struct State *StateList;
  struct StateVectorRow * list, *MakeEquations();
  extern char * ProgramName;
  extern int NumberOfStateBits, NumberOfInputs, StateVectorSize;
  char *DependsList, *calloc();

  Eliminated=0;
  DependsList = calloc(NumberOfInputs+NumberOfStateBits, sizeof(char));
  PrintInputNodes();
  for(index=NumberOfStateBits+NumberOfInputs; index<StateVectorSize; index++)
  {/*
    * Find the name of the output variable to be processed next.
    */
    name=NameFromTypeNumber(OUTPUTS, index);
    if(name==NULL)
    { fprintf(stderr,
      "%s(PrintSlang):  Output %d not found\n", ProgramName, index);
      exit(1);
    }
    printf("(def_fsm_output\n");
    printf("                   %s\n", name);
    printf("                   output-clock\n");
    list=MakeEquations(index);	/* Create a list of vectors for this output*/
    AlreadyFoundOne=FALSE;

#ifdef COLLAPSEBUG
/* This logic has not been converted to work for meally machines.....
*/

 /*
  * A 'next state' bit is a function of the current state and inputs.
  * A regular output bit is a function only of the current state bits.
  */
    if(index<NumberOfInputs+2*NumberOfStateBits) /*A 'next state' bit*/
    {
       Eliminated=Eliminated+
         Collapse(list, 0, NumberOfInputs+NumberOfStateBits-1, 1);
    }
    else
    {  
       Eliminated=Eliminated+
         Collapse(list, NumberOfInputs, NumberOfInputs+NumberOfStateBits-1, 1);
    }
#endif

    if(list==NULL)  /*No equation for this output*/
    {  /*fprintf(stderr,
	 "%s(PrintSlang):  Output %s is never set.\n", ProgramName, name);*/
       printf("OFF");
    }
    else {
	/* Initialize the DependsList for this node */
  	for (Dindex=0; Dindex<NumberOfInputs+NumberOfStateBits; Dindex++) 
		DependsList[Dindex] = ZERO;
    	printf("                   (Or\n");
    	while(list!=NULL) {
		/* Output bits for each vector */
       		PrintPterms(list, DependsList);
       		list=list->next;
    	}
    }
    printf("                   )\n"); /*End of expression for node def*/
    PrintDepends(DependsList);
    printf(")\n");
  }
  printf("\n\n");
  return(Eliminated);
} /*PrintSlang*/



PrintPterms(vector, DependsList)
/*********************************************************************
 * Scan all of the input bits for this vector.  If any of them are ONE
 * or ZERO then output "(And signal1 signal2 ...)"
 * For signals that are zero output "(Not signal1)" instead of signal.
 *********************************************************************
 */
   struct StateVectorRow *vector;
   char *DependsList;
{ int BitIndex, AlreadyFoundSomething;
  char Value, *signal, *NameFromTypeNumber();
  extern int NumberOfInputs, NumberOfStateBits;
  extern char * ProgramName;

  if(vector==NULL)
  {  fprintf(stderr, "%s(PrintPterms):  Null input\n", ProgramName);
     exit(1);
  }
  AlreadyFoundSomething=FALSE;
  for(BitIndex=0; BitIndex<NumberOfInputs+NumberOfStateBits; BitIndex++)
  {  signal=NameFromTypeNumber(INPUTS, BitIndex);
   /*printf("Output considered for signal %s\n", signal);*/
     if(signal==NULL)
     {  fprintf(stderr,
	"%s(PrintPterms):  Can't find input[%d]\n", ProgramName, BitIndex);
	exit(1);
     }
     Value=ReadBit(vector, signal);
     if(Value==ONE)
     {  
	if( ! AlreadyFoundSomething)
    		printf("                        (And  ");
	printf(" %s", signal);
	AlreadyFoundSomething=TRUE;
	DependsList[BitIndex] = ONE;
     }
     else
     if(Value==ZERO)
     {
	if( ! AlreadyFoundSomething)
    		printf("                        (And  ");
	printf(" (Not %s)", signal);
	AlreadyFoundSomething=TRUE;
	DependsList[BitIndex] = ONE;
     }
     else
     if(Value!=DONTCARE)
     {  fprintf(stderr,
        "%s(PrintPterms):  Illegal Value '%c'\n", ProgramName, Value);
	exit(1);
     }
  }
  if(AlreadyFoundSomething==FALSE)
     printf("ON");
  printf(")\n");
} /*PrintPterms*/

PrintInputNodes()
/***************************************************************************
**	Print the slang node definitions for each of the input signals.
***************************************************************************/
{
	int i;
	char *name;
	extern int NumberOfStateBits, NumberOfInputs;
	extern char * ProgramName;

	for(i=0; i<NumberOfInputs+NumberOfStateBits; i++) {
		/* For each input signal */
		name = NameFromTypeNumber(INPUTS, i);
    		if(name==NULL) {
			fprintf(stderr, 
				"%s(PrintInputNodes):  Output %d not found\n", 
				ProgramName, i);
      			exit(1);
    		}
		printf("(def_fsm_input %s input-clock)\n", name);
	}
}


PrintDepends(DependsList)
/****************************************************************************
**	Print the list of states that the node depends upon
*****************************************************************************/
char *DependsList;
{
	int i;
	extern int NumberOfStateBits, NumberOfInputs;

	printf("                   (depends output-clock ");
	for (i=0; i<NumberOfInputs+NumberOfStateBits; i++) {
		if (DependsList[i] == ONE)
			printf(" %s", NameFromTypeNumber(INPUTS, i));
	}
    	printf(")\n");
}


#ifdef LATER
PrintSymbolicState(name)
/*
 * Search every state for a vector which goes to the given state.
 * Print a (And ...) line for each vector.
 *
 */
   char * name;
{ 
  extern struct State *StateList;
  struct State *CurrentState;
  struct StateVectorRow *CurrentVector, *temp, *CopyVector();

  CurrentState=StateList;
  while(CurrentState!=NULL) /*Search every state for a vector...*/
  { CurrentVector=CurrentState->vectors;
    while(CurrentVector!=NULL)
    {  if(strcmp(CurrentVector->StateName, name) == 0)
       { /* NextState of current vector is name */
	 printf("   (And ");
       }
       CurrentVector=CurrentVector->next;
    }
    CurrentState=CurrentState->next;
  }
} /*PrintSymbolicState*/
#endif
