/*
 * MPLA - a tile based PLA generator
 *
 * Copyright (C) January 1983 by Robert N. Mayo, Berkeley, CA
 *
 * sccsid "@(#)mpla.c	1.13 	MPLA (Berkeley) 11/14/85";
 *
 */

#include "mpack.h"		/* ~cad/lib/mpack.h */
#include <stdio.h>
#include <string.h>
#include "stack.h"

/* Common Constants
 */
#define	TRUE	1
#define	FALSE	0
#define	ODD(x)	(x & 1)
#define	EVEN(x)	(!(x & 1))
#define MAXINT	32767
#define MININT	-32767
#define	max(a,b)	(a<b?b:a)
#define	min(a,b)	(a>b?b:a)
typedef int BOOL;

/* Parameters to this program
 */
#define TEMPLATE_LOC	"~cad/lib/mpla/p" /* the standard place for templates */
#define MAX_INPUTS	200
#define MAX_OUTPUTS	200
#define STRLEN		200

/* external routines
 */
/* (handled by string.h and stdio.h) */
/* extern char *sprintf(), *calloc(), *strcpy(); */


/* Global flags and variables
 */
char prog_name[STRLEN];	  /* the name of this program, for error messages */
int num_inputs = 0, num_pterms = 0, num_outputs = 0;
BOOL clk_inputs = FALSE, clk_outputs = FALSE;
BOOL comp_gnd = TRUE;		/* compute the extra ground rows and columns? */
BOOL comp_stretch = TRUE;	/* compute how much to stretch gnd and vdd? */
BOOL verbose = FALSE;
int gndsp = MAXINT, gndsp_and, gndsp_or;  /* spacings between ground lines */
int gnd_stretch = 0;

BOOL input_used[MAX_INPUTS*2];	/* keeps track of which inputs are used */
QUEUE inlab_q, outlab_q;  /* queues of labels for the inputs and outputs */
char symbolName[STRLEN];
char *MagicCompileTime,*MagicVersion;


/* tiles for the horizontal (AND plane) ground grid */
TILE THGleft_and, THGlx_and, THGrx_and, THGl_and, THGr_and, THG_mid, THG_or;
TILE THGright_or;

/* tiles for the vertical (NOR plane) ground grid */
TILE TVGtop_or, TVGux_or, TVGdx_or, TVGu_or, TVGd_or, THVG_or, TVGbot_or;

/* points to control overlap with the core of the 2 planes */
POINT Oleft_and, Oright_and, Otop_and, Obot_and;
POINT Otop_mid, Obot_mid;
POINT Oleft_or, Oright_or, Otop_or, Obot_or;

/* tile to set the vertical pitch of the AND and OR planes, as well as
 * the horizontal pitch of the AND plane
 */
TILE Tsp_and;

/* tile to set the horizontal pitch of the OR plane */
TILE Tsp_or;

/* tiles surrounding the AND plane */
TILE Tleftu_and, Tleftd_and, Tbot_and, Ttop_and;
TILE Tul_and, Tll_and;

/* tiles between the two planes */
TILE Tmidd, Tmidu;
TILE Ttop_mid, Tbot_mid;

/* tiles surroundint the OR plane */
TILE Trightd_or, Trightu_or;
TILE Ttopl_or, Ttopr_or, Tur_or;
TILE Tbotl_or, Tbotr_or, Tlr_or;

/* tiles within the AND plane */
TILE Tl_and, Tr_and, Tl0_and, Tr0_and, Tl1_and, Tr1_and;

/* tiles within the OR plane */
TILE Tu_or, Td_or, Tu0_or, Td0_or, Tu1_or, Td1_or;

TILE Tpla;



extern char get_ch();  /* get a (possibly pushed back) character */
extern unget_ch();	/* push a character back into the input */
extern char get_nonwhite();
extern get_word();	/* get an input word (words are delimited 
			by white space) */
extern unget_word();
extern get_newline();
extern char lower();
extern lower_word();

/*--------------------------------------------------------------------------
 *	initialize
 *
 *	Initializes global data structures.
 *
 *	Results:  none.
 *
 *	Side Effects:  global data structures are initialized.
 *-------------------------------------------------------------------------
 */
initialize()
{
    int i;

    for (i = 0; i < (MAX_INPUTS*2); i++)
	input_used[i] = FALSE;

    strcpy(symbolName, "mpla-out");
}



/*--------------------------------------------------------------------------
 *	error
 *
 *	Prints and error message and aborts if necessary.
 *
 *	Results:  none.
 *
 *	Side Effects:  error message and possible abort.
 *-------------------------------------------------------------------------
 */
error(num, str)
int num;
char *str;
{
    fprintf(stderr, "%s: %s.\n", prog_name, str);
    if (num > 0) {
	exit(num);
    }
}


/*--------------------------------------------------------------------------
 *	get_arg_num
 *
 *	gets a numerical argument from the command line
 *
 *	Results:  an integer.
 *
 *	Side Effects:  an error message if no number exists.
 *		       The number is removed from the command line.
 *-------------------------------------------------------------------------
 */
int get_arg_num(argc, argv, i)
int argc;
char *argv[];
int i;
{
    int j;

    if ((i >= argc) || (sscanf(argv[i], "%d", &j) != 1))
	error(1, "Number expected after command line option");
    argv[i][0] = '\0';
    return(j);
}


/*--------------------------------------------------------------------------
 *	options
 *
 *	process the command line and process the options
 *
 *	Results:  none.
 *
 *	Side Effects:  global flags and variables are set.
 *-------------------------------------------------------------------------
 */
options(argc, argv)
int argc;
char *argv[];
{
    int i, j;
    char *arg;

    strcpy(prog_name, argv[0]);
    i = 1;
    while (i < argc) {
	arg = argv[i];
	
	if (arg[0] == '-') {
	    for (j = 1; arg[j] != '\0'; j++) {
		switch (arg[j]) {
		    case 'G':
			comp_gnd = FALSE;
			gndsp = get_arg_num(argc, argv, i+1);
			if (gndsp < 1) gndsp = MAXINT;
			arg[j] = ' ';
			continue;
		    case 'I':
			clk_inputs = TRUE;
			arg[j] = ' ';
			continue;
		    case 'O':
			clk_outputs = TRUE;
			arg[j] = ' ';
			continue;
		    case 'V':
			verbose = TRUE;
			arg[j] = 'v';
			continue;
		    case 'S':
			comp_stretch = FALSE;
			gnd_stretch = get_arg_num(argc, argv, i+1);
			/* convert to a positive number of half-lambda */
			gnd_stretch = max(0, gnd_stretch*2);
			arg[j] = ' ';
			continue;
		}
	    }
	}
	i++;
    } /* while */

    if (verbose) {
	fprintf(stderr, "Clock inputs : %d\n", clk_inputs);
	fprintf(stderr, "Clock outputs : %d\n", clk_outputs);
	fprintf(stderr, "Compute gnd: %d\n", comp_gnd);
	fprintf(stderr, "Compute stretch: %d\n", comp_stretch);
	fprintf(stderr, "Gnd stretch: %d\n", gnd_stretch);
    }
} /* options */


/*--------------------------------------------------------------------------
 *	queue_labels
 *
 *	Reads in a line of input containing labels and stores them in a queue.
 *
 *	Results:  none.
 *
 *	Side Effects:  an input line is read and a queue is filled up.
 *-------------------------------------------------------------------------
 */
queue_labels(q)
QUEUE *q;
{
    char *str;
    char wrd[STRLEN];

    /* empty out the queue */
    while (!Q_isempty(*q)) {
	str = Q_pop(q);
	cfree(str);
    }

    /* put in 1 lines worth of labels */
    get_word(wrd);
    while (strcmp(wrd, "\n") != 0) {
	str = (char *) calloc(1, (strlen(wrd) + 5) * sizeof(char));
	(void) strcpy(str, wrd);
	Q_push(q, str);
	get_word(wrd);
    }
    unget_word(wrd);
}


/*--------------------------------------------------------------------------
 *	file_header
 *
 *	Process the header on the input truth table.
 *
 *	Results:  none.
 *
 *	Side Effects:  Global flags and variables are set.
 *-------------------------------------------------------------------------
 */
file_header()
{
    char wrd[STRLEN];

    get_word(wrd);
    while (wrd[0] == '.') {
	lower_word(wrd);
	if ( (strcmp(wrd, ".i") == 0) || (strcmp(wrd, ".il") == 0) )
	    num_inputs = get_num();
	else if ( (strcmp(wrd, ".o") == 0) || (strcmp(wrd, ".ol") == 0) )
	    num_outputs = get_num();
	else if (strcmp(wrd, ".p") == 0)
	    num_pterms = get_num();
	else if ( (strcmp(wrd, ".it") == 0) || (strcmp(wrd, ".ib") == 0) ||
	  (strcmp(wrd, ".ilt") == 0) || (strcmp(wrd, ".ilb") == 0) ) {
	    queue_labels(&inlab_q);
	} else if ( (strcmp(wrd, ".ot") == 0) || (strcmp(wrd, ".ob") == 0) ||
	  (strcmp(wrd, ".olt") == 0) || (strcmp(wrd, ".olb") == 0) ) {
	    queue_labels(&outlab_q);
	} else if ( strcmp(wrd, ".na") == 0 ) {
	    get_word(wrd);
	    if (strlen(wrd) <= 0)
		error(1, "The '.na' directive must supply a symbol name");
	    strcpy(symbolName, wrd);
	    if (verbose) 
		fprintf(stderr, 
			"CIF output symbol will (would) be named '%s'.\n", 
			symbolName);
	} else {
	    char msg[STRLEN];
	    (void) sprintf(msg, "The \"%s\" directive is not understood by this program", wrd);
	    error(0, msg);
	}
	get_newline();
	get_word(wrd);
    }
    unget_word(wrd);

    if (num_inputs > MAX_INPUTS)
	error(1, "Pla has too many inputs");
    if (num_outputs > MAX_OUTPUTS)
	error(1, "Pla has too many outputs");
    if (num_inputs <= 0)
	error(1, "Pla has no inputs");
    if (num_outputs <= 0)
	error(1, "Pla has no outputs");
    if (num_pterms <= 0)
	error(1, "Pla has no product terms");

    if (verbose) {
	fprintf(stderr, "Number of inputs: %d\n", num_inputs);
	fprintf(stderr, "Number of outputs: %d\n", num_outputs);
	fprintf(stderr, "Number of product terms: %d\n", num_pterms);
    }
}



/*--------------------------------------------------------------------------
 *	x_only
 *
 *	Takes a rectangle and returns a point containing the X dimension of
 *	the rectangle.
 *
 *	Results:  a point.
 *
 *	Side Effects:  none.
 *-------------------------------------------------------------------------
 */
POINT x_only(r)
RECTANGLE r;
{
    POINT p;

    p.x = r.x_right - r.x_left;
    p.y = 0;
    return p;
}



/*--------------------------------------------------------------------------
 *	y_only
 *
 *	Takes a rectangle and returns a point containing the Y dimension of
 *	the rectangle.
 *
 *	Results:  a point.
 *
 *	Side Effects:  none.
 *-------------------------------------------------------------------------
 */
POINT y_only(r)
RECTANGLE r;
{
    POINT p;

    p.x = 0;
    p.y = r.y_top - r.y_bot;
    return p;
}


/*--------------------------------------------------------------------------
 *	default_get
 *
 *	Return the size of the tile with the given name, or a null size if 
 *	it doesn't exist.
 *
 *	Results:  a rectangle.
 *
 *	Side Effects:  none.
 *-------------------------------------------------------------------------
 */
RECTANGLE default_get(s)
char *s;
{
    if (TPtile_exists(s))
	return TPsize_of_tile(TPname_to_tile(s));
    else
	return origin_rect;
}


/*--------------------------------------------------------------------------
 *	fetch_tiles
 *
 *	Cache all the tiles into variables of type TILE for future use.
 *
 *	Results:  none.
 *
 *	Side Effects:  many global variables of type TILE are set.
 *-------------------------------------------------------------------------
 */
fetch_tiles()
{
    BOOL any_iclk = FALSE;
    BOOL any_oclk = FALSE;

    /* horizontal ground (HG) tiles */
    THGleft_and = TPname_to_tile("HGleft-and");
    THGl_and = TPname_to_tile("HGl-and");
    THGr_and = TPname_to_tile("HGr-and");
    THGlx_and = TPname_to_tile("HGl.-and");
    THGrx_and = TPname_to_tile("HGr.-and");
    THG_mid = TPname_to_tile("HG-mid");
    THG_or = TPname_to_tile("HG-or");
    THGright_or = TPname_to_tile("HGright-or");

    /* vertical ground (VG) tiles */
    TVGux_or = TPname_to_tile("VGu-or");
    TVGdx_or = TPname_to_tile("VGd-or");
    TVGu_or = TPname_to_tile("VGu.-or");
    TVGd_or = TPname_to_tile("VGd.-or");
    THVG_or = TPname_to_tile("HVG-or");
    if (clk_outputs & TPtile_exists("CVGtop-or")) {
	TVGtop_or = TPname_to_tile("CVGtop-or");
	any_oclk = TRUE;
    } else
	TVGtop_or = TPname_to_tile("VGtop-or");
    if (clk_outputs & TPtile_exists("CVGbot-or")) {
	TVGbot_or = TPname_to_tile("CVGbot-or");
	any_oclk = TRUE;
    } else
	TVGbot_or = TPname_to_tile("VGbot-or");

    /* overlap (O) tiles */
    Otop_and = y_only(default_get("Otop-and"));
    Obot_and = y_only(default_get("Obot-and"));
    Oleft_and = x_only(default_get("Oleft-and"));
    Oright_and = x_only(default_get("Oright-and"));

    Otop_or = y_only(default_get("Otop-or"));
    Obot_or = y_only(default_get("Obot-or"));
    Oleft_or = x_only(default_get("Oleft-or"));
    Oright_or = x_only(default_get("Oright-or"));

    Otop_mid = y_only(default_get("Otop-mid"));
    Obot_mid = y_only(default_get("Obot-mid"));

    /* OR plane tiles */
    if (clk_outputs & TPtile_exists("Cbotl-or")) {
	Tbotl_or = TPname_to_tile("Cbotl-or");
	any_oclk = TRUE;
    } else
	Tbotl_or = TPname_to_tile("botl-or");
    if (clk_outputs & TPtile_exists("Cbotr-or")) {
	Tbotr_or = TPname_to_tile("Cbotr-or");
	any_oclk = TRUE;
    } else
	Tbotr_or = TPname_to_tile("botr-or");
    if (clk_outputs & TPtile_exists("Ctopl-or")) {
	Ttopl_or = TPname_to_tile("Ctopl-or");
	any_oclk = TRUE;
    } else
	Ttopl_or = TPname_to_tile("topl-or");
    if (clk_outputs & TPtile_exists("Ctopr-or")) {
	Ttopr_or = TPname_to_tile("Ctopr-or");
	any_oclk = TRUE;
    } else
	Ttopr_or = TPname_to_tile("topr-or");
    if (clk_outputs & TPtile_exists("Cur-or")) {
	Tur_or = TPname_to_tile("Cur-or");
	any_oclk = TRUE;
    } else
	Tur_or = TPname_to_tile("ur-or");
    if (clk_outputs & TPtile_exists("Clr-or")) {
	Tlr_or = TPname_to_tile("Clr-or");
	any_oclk = TRUE;
    } else
	Tlr_or = TPname_to_tile("lr-or");

    Trightd_or = TPname_to_tile("rightd-or");
    Trightu_or = TPname_to_tile("rightu-or");

    Tsp_or = TPname_to_tile("sp-or");
    Tu_or = TPname_to_tile("u.-or");
    Tu0_or = TPname_to_tile("u0-or");
    Tu1_or = TPname_to_tile("u1-or");
    Td_or = TPname_to_tile("d.-or");
    Td0_or = TPname_to_tile("d0-or");
    Td1_or = TPname_to_tile("d1-or");

    /* AND plane tiles */
    if (clk_inputs & TPtile_exists("Cul-and")) {
	Tul_and = TPname_to_tile("Cul-and");
	any_iclk = TRUE;
    } else
	Tul_and = TPname_to_tile("ul-and");
    if (clk_inputs & TPtile_exists("Cll-and")) {
	Tll_and = TPname_to_tile("Cll-and");
	any_iclk = TRUE;
    } else
	Tll_and = TPname_to_tile("ll-and");
    if (clk_inputs & TPtile_exists("Ctop-and")) {
	Ttop_and = TPname_to_tile("Ctop-and");
	any_iclk = TRUE;
    } else
	Ttop_and = TPname_to_tile("top-and");
    if (clk_inputs & TPtile_exists("Cbot-and")) {
	Tbot_and = TPname_to_tile("Cbot-and");
	any_iclk = TRUE;
    } else
	Tbot_and = TPname_to_tile("bot-and");
    Tleftd_and = TPname_to_tile("leftd-and");
    Tleftu_and = TPname_to_tile("leftu-and");

    Tsp_and = TPname_to_tile("sp-and");
    Tl_and = TPname_to_tile("l.-and");
    Tl0_and = TPname_to_tile("l0-and");
    Tl1_and = TPname_to_tile("l1-and");
    Tr_and = TPname_to_tile("r.-and");
    Tr0_and = TPname_to_tile("r0-and");
    Tr1_and = TPname_to_tile("r1-and");

    /* MID tiles */
    Ttop_mid = TPname_to_tile("top-mid");
    Tbot_mid = TPname_to_tile("bot-mid");

    Tmidd = TPname_to_tile("midd");
    Tmidu = TPname_to_tile("midu");

    /* error detection */
    if (clk_inputs && (!any_iclk) )
	error(0, "Warning -- The '-I' option had no effect");
    if (clk_outputs && (!any_oclk) )
	error(0, "Warning -- The '-O' option had no effect");
}


/*--------------------------------------------------------------------------
 *	compute_gnd_stretch
 *
 *	Figure out how much to stretch power and ground.
 *
 *	Results:  none.
 *
 *	Side Effects:  none.
 *-------------------------------------------------------------------------
 */
int compute_gnd_stretch()
{
    int l, width;

    /* caluclate ground width, calculation is based upon nMOS */
    /* this formula is stolen directly from the program 'mkpla' */
    l = TPget_lambda();
    width = (2000 * ((2*num_inputs) + num_pterms + (2*num_outputs))) / (l * l);

    /* convert to half lambda, and assume that the ground lines are 
     * 3 lambda wide 
     */
    return max(0, (width - 3)*2);
}


/*--------------------------------------------------------------------------
 *	do_stretch
 *
 *	Stretch a whole bunch to tiles by the specified amount.
 *
 *	Results:  none.
 *
 *	Side Effects:  tiles are stretched.
 *-------------------------------------------------------------------------
 */
do_stretch(l, stretch)
char *l;
int stretch;		/* number of half lambda to stretch */
{
    TPstretch_tile(Tul_and, l, stretch);
    TPstretch_tile(Tll_and, l, stretch);

    TPstretch_tile(Tur_or, l, stretch);
    TPstretch_tile(Tlr_or, l, stretch);

    TPstretch_tile(Tleftd_and, l, stretch);
    TPstretch_tile(Tleftu_and, l, stretch);
    TPstretch_tile(Ttop_and, l, stretch);
    TPstretch_tile(Tbot_and, l, stretch);

    TPstretch_tile(Trightd_or, l, stretch);
    TPstretch_tile(Trightu_or, l, stretch);
    TPstretch_tile(Ttopl_or, l, stretch);
    TPstretch_tile(Tbotl_or, l, stretch);
    TPstretch_tile(Ttopr_or, l, stretch);
    TPstretch_tile(Tbotr_or, l, stretch);

    TPstretch_tile(Ttop_mid, l, stretch);
    TPstretch_tile(Tmidu, l, stretch);
    TPstretch_tile(Tmidd, l, stretch);
    TPstretch_tile(Tbot_mid, l, stretch);

    TPstretch_tile(THGleft_and, l, stretch);
    TPstretch_tile(THG_mid, l, stretch);
    TPstretch_tile(THGright_or, l, stretch);

    TPstretch_tile(TVGtop_or, l, stretch);
    TPstretch_tile(TVGbot_or, l, stretch);
}


/*--------------------------------------------------------------------------
 *	make_tiles
 *
 *	Read in the tiles and stretch them.
 *
 *	Results:  none.
 *
 *	Side Effects:  Some global variables are filled in.
 *-------------------------------------------------------------------------
 */
make_tiles()
{
    int stretch;

    fetch_tiles();

    if (comp_stretch)
	stretch = compute_gnd_stretch();
    else
	stretch = gnd_stretch;
    if (stretch != 0) {
	do_stretch("<GND>", stretch);
	do_stretch("<Vdd>", stretch);
    }
}


/*--------------------------------------------------------------------------
 *	compute_gnd_mesh
 *
 *	Figure out how many ground lines to put in the AND and OR planes.
 *
 *	Results:  none.
 *
 *	Side Effects:  global variables are set.
 *-------------------------------------------------------------------------
 */
compute_gnd_mesh()
{
    int num_or, num_and;

    if (comp_gnd) {
	int l;
	l = TPget_lambda();
	/* These calculations where stolen from the program 'mkpla' */
	/* They are based upon a particluar nMOS pla */
	/* see ~cad/src/mkpla for details */
	gndsp = (l * l) / 625;	/* small-lambda limit */
	gndsp = min(gndsp, 27);  /* large-lambda limit */
	if (gndsp < 2) {
	    error(0, "Warning: PLA requires too many ground lines and may not work");
	    error(0, "        (lambda is too small)");
	    gndsp = 2;
	}
    } 

    /* place the ground lines evenly */
    num_or = (num_outputs-1) / gndsp;
    if (num_or == 0) {
	gndsp_or = MAXINT;
    } else {
	/* divide, rounding up */
	gndsp_or = (num_outputs + num_or) / (num_or + 1);
	/* round to multiple of 2 */
	gndsp_or = (min(gndsp, gndsp_or + 1) / 2) * 2;
    }
    num_and = (num_pterms-1) / gndsp;
    if (num_and == 0) {
	gndsp_and = MAXINT;
    } else {
	/* divide, rounding up */
	gndsp_and = (num_pterms + num_and) / (num_and + 1);
	/* round to multiple of 2 */
	gndsp_and = (min(gndsp, gndsp_and + 1) / 2) * 2;
    }

    if (verbose) {
	fprintf(stderr, "Gnd spacing: %d\n", gndsp);
	fprintf(stderr, "%d ground(s) in the AND plane, spaced %d apart\n", 
	  num_and, gndsp_and);
	fprintf(stderr, "%d ground(s) in the OR plane, spaced %d apart\n", 
	  num_or, gndsp_or);
    }
}

/*--------------------------------------------------------------------------
 *	enclose
 *
 *	Find the minimum bounding box for two rectangles.
 *
 *	Results:  a rectangle.
 *
 *	Side Effects:  none.
 *-------------------------------------------------------------------------
 */
RECTANGLE enclose(r, s)
RECTANGLE r, s;
{
    RECTANGLE g;

    g.x_left = min(r.x_left, s.x_left);
    g.x_right = max(r.x_right, s.x_right);
    g.y_bot = min(r.y_bot, s.y_bot);
    g.y_top = max(r.y_top, s.y_top);
    return g;
}


/*--------------------------------------------------------------------------
 *	get_minterm
 *
 *	Read in a minterm from the truth table file.
 *
 *	Results:  two strings -- one for each plane.
 *
 *	Side Effects:  the input file is advanced.
 *-------------------------------------------------------------------------
 */
get_minterm(sand, sor)
char *sand;		/* will be filled in with the and terms */
char *sor;		/* will be filled in with the or terms */
{
    int input, ii, output;
    char ch;

    ch = get_nonwhite();
    while (ch == '.') {
	char msg[STRLEN], wrd[STRLEN];
	unget_ch(ch);
	get_word(wrd);
	(void) sprintf(msg, "'%s' directive ignored while reading truth table",
	  wrd);
	error(0, msg);
	get_newline();
	ch = get_nonwhite();
    }
    ii = 0;
    for (input = 1; input <= num_inputs; input++) {
	switch (ch) {
	    case '1':
		sand[ii] = '1';
		input_used[ii] = TRUE;
		break;
	    case '0':
	    case '-':
	    case 'x':
	    case 'X':
		if (input_used[ii])
		    sand[ii] = '0';
		else
		    sand[ii] = '.';
		break;
	    case '\n':
		error(1, "Format error in input -- product term is too short");
		break;
	    default:
		error(1, "Strange character in minterm description");
		break;
	}
	ii++;
	switch (ch) {
	    case '0':
		sand[ii] = 'I';
		input_used[ii] = TRUE;
		break;
	    case '1':
	    case '-':
	    case 'x':
	    case 'X':
		if (input_used[ii])
		    sand[ii] = 'O';
		else
		    sand[ii] = ',';
		break;
	}
	ii++;
	ch = get_nonwhite();
    }
    sand[ii] = '\0';

    ii = 0;
    for (output = 1; output <= num_outputs; output++) {
	switch (ch) {
	    case '1':
		sor[ii] = '1';
		break;
	    case '0':
	    case '-':
	    case 'x':
	    case 'X':
		sor[ii] = '0';
		break;
	    case '\n':
		error(1, "Format error in input -- product term is too short");
		break;
	    default:
		error(1, "Strange character in minterm description");
		break;
	}
	ii++;
	ch = get_nonwhite();
    }

    sor[ii] = '\0';
    for(ii--; ii > 0; ii--) {
	if (sor[ii] == '0')
	    sor[ii] = '.';
	else
	    break;
    }

    if (verbose) fprintf(stderr, "Minterm: %s-%s\n", sand, sor);
    if (ch != '\n')
	error(1, "Format error in input -- product term is too long");
}


/*--------------------------------------------------------------------------
 *	paint_element
 *
 *	Paint an element of a PLA row, and then slide over some.
 *
 *	Results:  a new rectangle to be aligned with.
 *
 *	Side Effects:  none.
 *-------------------------------------------------------------------------
 */
RECTANGLE paint_element(t, s, c)
RECTANGLE c;
TILE t,s;
{
    RECTANGLE r;

    r = TPpaint_tile(t, Tpla, align(rLR(c), tLL(t)));
    r = TPdisp_tile(s, align(rLR(c), tLL(s)));
    return r;
}



/*--------------------------------------------------------------------------
 *	label_io
 *
 *	Look for 1 input or output terminal in the PLA, and label it.
 *
 *	Results:  none.
 *
 *	Side Effects:  a label is put in the PLA.
 *-------------------------------------------------------------------------
 */
label_io(inputnum, outputnum, r)
int *outputnum, *inputnum;
RECTANGLE r;
{
    RECTANGLE loc;

    if (TPfind_label(Tpla, &r, "<input>", &loc)) {
	TPremove_labels(Tpla, "<input>", loc);
	(*inputnum)++;
	if (Q_isempty(inlab_q)) {
	    char str[STRLEN];
	    (void) sprintf(str, "input_%d", *inputnum);
	    TPplace_label(Tpla, loc, str);
	} else {
	    char *cp;
	    cp = Q_pop(&inlab_q);
	    TPplace_label(Tpla, loc, cp);
	    cfree(cp);
	}
    }
    if (TPfind_label(Tpla, &r, "<output>", &loc)) {
	TPremove_labels(Tpla, "<output>", loc);
	(*outputnum)++;
	if (Q_isempty(outlab_q)) {
	    char str[STRLEN];
	    (void) sprintf(str, "output_%d", *outputnum);
	    TPplace_label(Tpla, loc, str);
	} else {
	    char *cp;
	    cp = Q_pop(&outlab_q);
	    TPplace_label(Tpla, loc, cp);
	    cfree(cp);
	}
    }
}



/*--------------------------------------------------------------------------
 *	paint_row
 *
 *	Paint 1 row of the PLA.
 *
 *	Results:  a rectangle to align the next row with.
 *
 *	Side Effects:  paint is added to the PLA.
 *-------------------------------------------------------------------------
 */
RECTANGLE paint_row(and_chars, or_chars, even, above, below, gndrow, r)
char *and_chars;
char *or_chars;
BOOL even, above, below, gndrow;
RECTANGLE r;
{
    int ik;
    int inputnum = 0, outputnum = 0;
    RECTANGLE ret, top, bot, cur, act;
    TILE this_tile;

    /* space down to next row */
    if (gndrow)
	cur = TPdisp_tile(THGleft_and, align(rLL(r), tUR(THGleft_and)) );
    else
	cur = TPdisp_tile(Tsp_and, align(rLL(r), tUR(Tsp_and)) );

    /* handle the pullups and such along the left side of the row */
    if (gndrow)
	cur = paint_element(THGleft_and, THGleft_and, cur);
    else if (even)
	cur = paint_element(Tleftd_and, Tleftd_and, cur);
    else
	cur = paint_element(Tleftu_and, Tleftu_and, cur);
    bot = cur;
    top = cur;
    ret = cur;
    if (above) 
        top = TPpaint_tile(Tul_and, Tpla, align(rUR(TPsub_rp(cur, Otop_and)), 
	  tLR(Tul_and)) );
    if (below) 
	bot = TPpaint_tile(Tll_and, Tpla, align(rLR(TPadd_rp(cur, Obot_and)), 
	  tUR(Tll_and)) );
    ret = enclose(ret, enclose(top, bot));
    cur = TPsub_rp(cur, Oleft_and);

    /* do the core of the AND part of the row */
    for (ik = 0; and_chars[ik] != '\0'; ik++) {
	if (gndrow) {
	    switch (and_chars[ik]) {
		case '0':
		case '1':
		    this_tile = THGl_and;
		    break;
		case 'O':
		case 'I':
		    this_tile = THGr_and;
		    break;
		case '.':
		    this_tile = THGlx_and;
		    break;
		case ',':
		    this_tile = THGrx_and;
		    break;
		default:
		    error(1, "mpla internal error (gnd-and)");
		    break;
	    }
	} else {
	    switch (and_chars[ik]) {
		case '1':
		    this_tile = Tl1_and;
		    break;
		case 'I':
		    this_tile = Tr1_and;
		    break;
		case '0':
		    this_tile = Tl0_and;
		    break;
		case 'O':
		    this_tile = Tr0_and;
		    break;
		case '.':
		    this_tile = Tl_and;
		    break;
		case ',':
		    this_tile = Tr_and;
		    break;
		default:
		    error(1, "mpla internal error (core-and)");
		    break;
	    }
	}
	act = TPdisp_tile(this_tile, align(rLR(cur), tLL(this_tile)));
	cur = paint_element(this_tile, Tsp_and, cur);
	if (EVEN(ik)) {
	    if (above) {
		top = TPpaint_tile(Ttop_and, Tpla, 
		  align(rUL(TPsub_rp(act, Otop_and)), tLL(Ttop_and)) );
		label_io(&inputnum, &outputnum, top);
	    }
	    if (below) {
		bot = TPpaint_tile(Tbot_and, Tpla, 
		  align(rLL(TPadd_rp(act, Obot_and)), tUL(Tbot_and)) );
		label_io(&inputnum, &outputnum, bot);
	    }
	}
    } /* for ik */

    cur = TPsub_rp(act, Oright_and);

    /* do the middle area between planes */
    if (gndrow)
	cur = paint_element(THG_mid, THG_mid, cur);
    else if (even)
	cur = paint_element(Tmidd, Tmidd, cur);
    else 
	cur = paint_element(Tmidu, Tmidu, cur);
    if (above)
	top = TPpaint_tile(Ttop_mid, Tpla, 
	  align(rUL(TPsub_rp(cur, Otop_mid)), tLL(Ttop_mid)));
    if (below) 
	bot = TPpaint_tile(Tbot_mid, Tpla, 
	  align(rLL(TPadd_rp(cur, Obot_mid)), tUL(Tbot_mid)));

    cur = TPsub_rp(cur, Oleft_or);

    /* do the OR plane */
    for (ik = 0; or_chars[ik] != '\0'; ik++) {
	if (gndrow) {
	    this_tile = THG_or;
	}
	else {
	    switch (or_chars[ik]) {
		case '0':
		    if (even)
			this_tile = Td0_or;
		    else
			this_tile = Tu0_or;
		    break;
		case '1':
		    if (even)
			this_tile = Td1_or;
		    else
			this_tile = Tu1_or;
		    break;
		case '.':
		    if (even)
			this_tile = Td_or;
		    else
			this_tile = Tu_or;
		    break;
		default:
		    error(1, "mpla internal error (core-or)");
		    break;
	    } /* case */
	}

	act = TPdisp_tile(this_tile, align(rLR(cur), tLL(this_tile)));
	cur = paint_element(this_tile, Tsp_or, cur);

	if (above) {
	    if (EVEN(ik))
		top = TPpaint_tile(Ttopl_or, Tpla,
		      align(rUL(TPsub_rp(act, Otop_or)), tLL(Ttopl_or)) );
	    else
		top = TPpaint_tile(Ttopr_or, Tpla,
		      align(rUR(TPsub_rp(act, Otop_or)), tLR(Ttopr_or)) );
	    label_io(&inputnum, &outputnum, top);
	}
	if (below) {
	    if (EVEN(ik))
		bot = TPpaint_tile(Tbotl_or, Tpla,
		      align(rLL(TPadd_rp(act, Obot_or)), tUL(Tbotl_or)) );
	    else
		bot = TPpaint_tile(Tbotr_or, Tpla,
		      align(rLR(TPadd_rp(act, Obot_or)), tUR(Tbotr_or)) );
	    label_io(&inputnum, &outputnum, bot);
	}

	if ( ( ((ik+1) % gndsp_or) == 0) && (or_chars[ik+1] != '\0')) {
	    TILE g_tile;
	    if (gndrow) {
		    g_tile = THVG_or;
	    } else if (or_chars[ik+1] == '.') {
		if (even)
		    g_tile = TVGd_or;
		else
		    g_tile = TVGu_or;
	    } else {
		if (even)
		    g_tile = TVGdx_or;
		else
		    g_tile = TVGux_or;
	    }
	    cur = paint_element(g_tile, g_tile, cur);

	    if (above) {
		top = TPpaint_tile(TVGtop_or, Tpla,
		      align(rUL(TPsub_rp(cur, Otop_or)), tLL(TVGtop_or)) );
	    }
	    if (below) {
		bot = TPpaint_tile(TVGbot_or, Tpla,
		      align(rLL(TPadd_rp(cur, Obot_or)), tUL(TVGbot_or)) );
	    }


	}
    } /* for ik */

    cur = TPsub_rp(cur, Oright_or);

    /* handle the tiles on the right side */
    {
	TILE right_t;
	if (gndrow)
	    right_t = THGright_or;
	else if (even)
	    right_t = Trightd_or;
	else
	    right_t = Trightu_or;
	cur = TPpaint_tile(right_t, Tpla, 
	  align(rLR(TPsub_rp(cur, Oright_or)), tLL(right_t)) );
    }

    if (above) {
	top = TPpaint_tile(Tur_or, Tpla, 
	  align(rUL(TPsub_rp(cur, Otop_or)), tLL(Tur_or)) );
    }
    if (below) {
	bot = TPpaint_tile(Tlr_or, Tpla, 
	  align(rLL(TPadd_rp(cur, Obot_or)), tUL(Tlr_or)) );
    }

    /* that's all folks */
    return ret;
} 


/*--------------------------------------------------------------------------
 *	paint_pla
 *
 *	Paint a PLA described by the truth table on the standard input
 *
 *	Results:  none.
 *
 *	Side Effects:  a PLA is generated.
 *-------------------------------------------------------------------------
 */
paint_pla(r)
RECTANGLE r;
{
    int pterm;
    RECTANGLE cur_rect;
    char s_and[MAX_INPUTS+1], s_or[MAX_OUTPUTS+1];

    cur_rect = r;

    for (pterm = 1; pterm <= num_pterms; pterm++) {
	if (verbose) fprintf(stderr, "#%d ", pterm);
	get_minterm(s_and, s_or);
    	cur_rect = paint_row(s_and, s_or, EVEN(pterm), (pterm == 1),
	  (pterm == num_pterms), FALSE, cur_rect);
	if ( ((pterm % gndsp_and) == 0) && (pterm != num_pterms) )
	    cur_rect = paint_row(s_and, s_or, TRUE, FALSE, FALSE, 
	      TRUE, cur_rect);
    }
    TPstrip_labels(Tpla, '<');
} /* paint_pla */


/*--------------------------------------------------------------------------
 *	main
 *
 *	Paint a PLA
 *
 *	Results:  none.
 *
 *	Side Effects:   a PLA is generated, files are modified.
 *-------------------------------------------------------------------------
 */
main(argc, argv)
int argc;
char *argv[];
{

    initialize();
    options(argc, argv);
    TPinitialize(argc, argv, TEMPLATE_LOC);
    file_header();
    Tpla = TPcreate_tile(symbolName);

    make_tiles();
    compute_gnd_mesh();
    paint_pla(origin_rect);

    TPwrite_tile(Tpla, "");
    exit(0);
}
