/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Checks.c	40.1      peg      6/29/88"               */

/*
 * Checks.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)Checks.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint


#include "LookupDefs.i"
extern FILE *SummaryFile;

CheckTable()
/************************************************************
 * Check the table for errors on input and errors in the code.
 ************************************************************
 */
{  CheckInputs();
   CheckCurrentState();
   CheckNextState();
   CheckOutputs();
} /*CheckTable*/


CheckInputs()
/********************************************************************
 * For each input, make sure that at least one vector wants it to be
 * 0, and one vector wants it to be a 1.
 ********************************************************************
 */
{ int index, FoundZero, FoundOne;
  char *name, ReadBit(), *NameFromTypeNumber(), ch;
  extern struct State *StateList;
  extern char * ProgramName;
  struct State *CurrentState;
  struct StateVectorRow *CurrentVector;
  extern int NumberOfInputs;

  for(index=0; index<NumberOfInputs; index++)
  { name=NameFromTypeNumber(INPUTS, index);
    FoundZero=FALSE;
    FoundOne =FALSE;
    if(name==NULL)
    { fprintf(stderr, "%s(CheckInputs):  Input %d not found\n", ProgramName);
      exit(1);
    }
    CurrentState=StateList;
    while(CurrentState!=NULL) /*Search every state for a vector...*/
    { CurrentVector=CurrentState->vectors;
      while(CurrentVector!=NULL)
      {  ch=ReadBit(CurrentVector, name);
	 if(ch==ONE) /*...which uses input value ONE*/
	    FoundOne=TRUE;
	 else
	 if(ch==ZERO) /*...which uses input value ZERO*/
	    FoundZero=TRUE;
         CurrentVector=CurrentVector->next;
      }
      CurrentState=CurrentState->next;
    }

 /* Missing specification for this input
  */ if((FoundZero==FALSE)||(FoundOne==FALSE))
       fprintf(stderr,
          "%s(CheckInputs):  Warning:  Input %s incompletely used.\n",
	  ProgramName, name);
  }
} /*CheckInputs*/


CheckNextState()
/********************************************************************
 * For each state, make sure that a next state is specified.  If the
 * next state bits are don't cares, then complain.
 ********************************************************************
 */
{ int index, FoundError, LsbIndex;
  char *name, ReadBit(), *NameFromTypeNumber(), ch;
  extern struct State *StateList;
  extern char * ProgramName;
  struct State *CurrentState;
  struct StateVectorRow *CurrentVector;
  extern int NumberOfInputs, NumberOfStateBits;

  LsbIndex=NumberOfInputs+NumberOfStateBits;

  CurrentState=StateList;
  while(CurrentState!=NULL) /*Search every state*/
  {  FoundError=FALSE;
     CurrentVector=CurrentState->vectors;
     while(CurrentVector!=NULL) /*Search every vector in the state*/
     {/*
       * Check every bit of the NEXT STATE field for this vector
       */
	for(index=LsbIndex; index<LsbIndex+NumberOfStateBits; index++)
        {  name=NameFromTypeNumber(OUTPUTS, index);
           if(name==NULL)
           { fprintf(stderr,
	     "%s(CheckNextState):  Output %d not found\n", ProgramName, index);
             exit(1);
           }
           ch=ReadBit(CurrentVector, name);
	   if((ch!=ONE)&&(ch!=ZERO)) /*Next state not specified*/
	      FoundError=TRUE;
	}
        CurrentVector=CurrentVector->next;
     }
     if(FoundError)
        fprintf(stderr, "%s(CheckNextState):  Bad next state for state %s.\n",
	        ProgramName, CurrentState->name);
     CurrentState=CurrentState->next;
  }
} /*CheckNextState*/


CheckCurrentState()
/**********************************************************************
 * For each state, make sure that a current state is properly specified.
 * If the current state bits are don't cares, then complain.
 ********************************************************************
 */
{ int index, FoundError, LsbIndex;
  char *name, ReadBit(), *NameFromTypeNumber(), ch;
  extern struct State *StateList;
  extern char * ProgramName;
  struct State *CurrentState;
  struct StateVectorRow *CurrentVector;
  extern int NumberOfInputs, NumberOfStateBits;

  LsbIndex=NumberOfInputs;

  CurrentState=StateList;
  while(CurrentState!=NULL) /*Search every state*/
  {  FoundError=FALSE;
     CurrentVector=CurrentState->vectors;
     while(CurrentVector!=NULL) /*Search every vector in the state*/
     {/*
       * Check every bit of the CURRENT STATE field for this vector
       */
	for(index=LsbIndex; index<LsbIndex+NumberOfStateBits; index++)
        {  name=NameFromTypeNumber(INPUTS, index);
           if(name==NULL)
           { fprintf(stderr,
	    "%s(CheckCurrentState):  Input %d not found\n", ProgramName, index);
             exit(1);
           }
           ch=ReadBit(CurrentVector, name);
	   if((ch!=ONE)&&(ch!=ZERO)) /*Next state not specified*/
	      FoundError=TRUE;
	}
        CurrentVector=CurrentVector->next;
     }
     if(FoundError)
     fprintf(stderr, "%s(CheckCurrentState): Bad current state for state %s.\n",
	       ProgramName, CurrentState->name);
     CurrentState=CurrentState->next;
  }
} /*CheckCurrentState*/


CheckOutputs()
/********************************************************************
 * For each output, make sure that at least one vector wants it to be
 * a 1.
 ********************************************************************
 */
{ int index, FoundOne, FirstOutput;
  char *name, ReadBit(), *NameFromTypeNumber(), ch;
  extern char * ProgramName;
  extern struct State *StateList;
  struct State *CurrentState;
  struct StateVectorRow *CurrentVector;
  extern int NumberOfInputs, NumberOfOutputs, NumberOfStateBits;

  FirstOutput=NumberOfInputs+2*NumberOfStateBits;

  for(index=FirstOutput; index<FirstOutput+NumberOfOutputs; index++)
  { FoundOne =FALSE;
    name=NameFromTypeNumber(OUTPUTS, index);
    if(name==NULL)
    { fprintf(stderr,
      "%s(CheckOutputs):  Output %d not found\n", ProgramName, index);
      exit(1);
    }
    CurrentState=StateList;
    while(CurrentState!=NULL) /*Search every state for a vector...*/
    { CurrentVector=CurrentState->vectors;
      while(CurrentVector!=NULL)
      {  ch=ReadBit(CurrentVector, name);
	 if(ch==ONE) /*...which uses input value ONE*/
	    FoundOne=TRUE;
         CurrentVector=CurrentVector->next;
      }
      CurrentState=CurrentState->next;
    }
    if(FoundOne==FALSE) /*Missing specification for this input*/
    fprintf(stderr, "%s(CheckOutputs): Warning: Output %s is never set.\n",
       ProgramName, name);
  }
} /*CheckOutputs*/
