/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Init.c	40.1      peg      6/29/88"               */

/*
 * Init.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)Init.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint


#include <stdio.h>
#include "LookupDefs.i"
FILE *SummaryFile;

BitsPerState()
/**********************************************************
 * Return the number of bits in the state field of the fsm.
 **********************************************************
 */
{ int result, temp;
  extern int NumberOfStates;
  extern char * ProgramName;

  if(NumberOfStates<=1)
  { fprintf(stderr, "%s(BitsPerState):  Invalid number of states=%d.\n",
       ProgramName, NumberOfStates);
    exit(1);
  };
  temp=NumberOfStates-1;
  result=0;
  while(temp>0)
  {  result++;
     temp=temp>>1;
  }
  return(result);
} /*BitsPerState*/


Init(argc, argv)
/***********************************************************
 * This must be called before any other processing gets done.
 ***********************************************************
 */
  int argc;
  char * argv[];
{ extern int NumberOfStates, NumberOfInputs, NumberOfOutputs;
  extern int InputCounter, OutputCounter, NowAdding;
  extern int TotalEliminated, StateOffset, TableFlag, SummaryFlag;
  extern char * StateBaseName, * ProgramName;
  extern struct DataItem *SymbolList;
  extern struct State *StateList;
  int loop, GotFileName;
  FILE *freopen(), *fopen();
  extern FILE *SummaryFile;
  char *InputFileName;

  ProgramName=argv[0];
  GotFileName=FALSE;
  InputFileName=NULL;
  StateOffset=0;
  StateBaseName="FirstState";
  TableFlag      =  0; /*Default to no table generation*/
  SummaryFlag    =  0; /*Default to no summary info    */
  TotalEliminated=  0;
  NumberOfInputs =  0;
  NumberOfOutputs=  0;
  NumberOfStates =  0;
  InputCounter   =  0;
  OutputCounter  =  0;
  NowAdding      =ILLEGAL;
  SymbolList     =NULL;
  StateList      =NULL;
  for(loop=1;loop<argc;loop++)
  {  if(argv[loop][0]=='-')
     {  switch(argv[loop][1])
	{ case 't':     TableFlag=1;
			break;
	  case 's':     SummaryFlag=1;
			break;
	  default :  fprintf(stderr,
		       "%s(Init):  Bad flag %s\n", ProgramName, argv[loop]);
		     break;
	}
     }
     else
/*
 *   The argument is the name of an input file for peg to use in place of
 *   standard input.  Use it if one has not yet been specified successfully.
 */
     if(GotFileName)
          fprintf(stderr, "%s(Init):  Command line argument '%s' ignored.\n",
		           ProgramName,                     argv[loop]);
     else
     {  if(freopen(argv[loop], "r", stdin)==NULL)
	{ fprintf(stderr, "%s(Init):  Can't open input file %s.\n",
		           ProgramName, 		    argv[loop]);
	  exit(1);
	}
	else
	GotFileName=TRUE;
	InputFileName=argv[loop];
     }
  }
  if(TableFlag||SummaryFlag)
  { if((SummaryFile=fopen("peg.summary", "w"))==NULL)
    { fprintf(stderr, "%s(Init): Can't ", ProgramName);
      fprintf(stderr, "open file peg.summary.\n");
      exit(1);
    }
    else
    { fprintf(SummaryFile, "SUMMARY INFORMATION GENERATED BY PEG FROM FILE ");
      if(GotFileName)
           fprintf(SummaryFile, "%s\n", InputFileName);
      else fprintf(SummaryFile, "<stdin>\n");
      fflush(SummaryFile);
    }
  }
} /*Init*/
