/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)States.c	40.1      peg      6/29/88"               */

/*
 * States.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)States.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint


#include "ParserDefs.i"
#include "LookupDefs.i"
#include "Externals.i"
extern FILE *SummaryFile;

struct State *
NewState(name)
/*******************************************************
 * Return a pointer to a defalt-initialized state record.
 *******************************************************
 */
    char *name;
{ struct State *temp;
  struct StateVectorRow *NewStateVector();
  extern char * ProgramName;
  char *calloc();

  temp=(struct State *) calloc(1, sizeof(struct State));
  if(temp==0)
  {  fprintf(stderr, "%s(NewState):  calloc failedNewState.\n", ProgramName);
     exit(1);
  };
  temp->name   =name;
  temp->vectors=NewStateVector();
  temp->next   =NULL;
  SetStateBits(temp->vectors, name, INPUTS);
  return(temp);
} /*NewState*/


struct State *
StateFromName(name)
/**************************************************************
 * Return a pointer to the state structure with the given name.
 **************************************************************
 */
   char *name;
{  struct State *temp;
   extern struct State *StateList;
   extern char * ProgramName;
   
   if(TypeFromName(name)!=STATE)
   {  fprintf(stderr, 
      "%s(StateFromName):  Identifier %s not a state\n", ProgramName, name);
      exit(1);
   }
   temp=StateList;
   for(;;)
   {  if(temp==NULL)
      {  fprintf(stderr,
	 "%s(StateFromName):  State %s not found\n", ProgramName, name);
	 exit(1);
      }
      else
      if(strcmp(name, temp->name)==0) break;
      else
      temp=temp->next;
   }
   return(temp);
} /*StateFromName*/


/*
StateBranch(From, On, IfZero, IfOne)
 *
 * Find the FROM state on the global StateList.  For each row vector
 * associated with the state, if the ON signal is currently DONTCARE
 * then split the row vector into two and branch to the IFZERO state
 * when ON=0 and branch to the IFONE state when ON=1.
 *
 * 
   char *From, *On, *IfZero, *IfOne;
{  struct State *CurrentState;
   struct StateVectorRow *temp, *new, *CopyVector();

   CurrentState=StateFromName(From);
   temp=CurrentState->vectors;
   while(temp!=NULL)
   {   new=CopyVector(temp);
       temp->next=new;
       SetBit(temp, On, ZERO);
       SetStateBits(temp, IfZero, OUTPUTS);
       SetBit(new, On, ONE);
       SetStateBits(new, IfOne, OUTPUTS);
       temp=new->next;
   }
}  *StateBranch*/


StateJumpOn(From, On, Into)
/*******************************************************************
 * Find the FROM state on the global StateList.  For each row vector
 * associated with the state, change the DONTCARE for the input
 * signal ON to 0.  Add an additional row vector which branches on
 * 1 into the INTO state.
 *******************************************************************
 */
   char *On, *Into;
   struct State *From;
{  struct StateVectorRow *temp, *new, *CopyVector();
   extern char * ProgramName;

   if(From==NULL)
   {  fprintf(stderr, "%s(StateJumpOn):  Null FROM, ON %s INTO state %s\n",
         ProgramName, On, Into);
      exit(1);
   }
   temp=From->vectors;
   while(temp!=NULL)
   {   SetBit(temp, On, ZERO);
       temp=temp->next;
   }
/*
 * Allocate a new state vector.  Copy outputs from the current state.
 */
   new=NewStateVector();
   CopyOutputs(new, From->vectors);
   SetStateBits(new, From->name, INPUTS);
   SetStateBits(new, Into, OUTPUTS);
   SetBit(new, On, ONE);
/*
 * Add the new record to the head of the vector list.
 */
   new->next=From->vectors;
   From->vectors=new;
} /*StateJumpOn*/


/*
StateJump(From, Into)
 ********************************************************************
 * Find the FROM state on the global StateList.  Set an unconditional
 * branch into the INTO state.  This is an error if there is more
 * than one row vector.
 ********************************************************************
 * 
   char *From, *Into;
{  struct State *CurrentState;
   extern char * ProgramName;

   CurrentState=StateFromName(From);
   if(CurrentState->vectors!=NULL)
   { if(CurrentState->vectors->next!=NULL)
     { fprintf(stderr, 
       "%s(StateBranch):  multiple vectors, state=%s\n", ProgramName, From);
       exit(1);
     }
     SetStateBits(CurrentState->vectors, Into, OUTPUTS);
   }
}  *StateJump*/


AddResets()
/**************************************************************
 * Add a branch to the reset state for each state, when it sees
 * a reset signal.
 **************************************************************
 */
{ extern struct State *StateList;
  struct State *CurrentState;

  CurrentState=StateList;
  while(CurrentState!=NULL)
  {  StateJumpOn(CurrentState, "RESET", StateList->name);
     CurrentState=CurrentState->next;
  }
} /*AddResets*/


MakeStateList()
/*************************************************************************
 * The last output has already been read.  Traverse the ParserList and
 * allocate a state on StateList for each StateRecord.
 *************************************************************************
 */
{ extern struct State *StateList;
  extern struct DataItem *SymbolList;
  extern int NumberOfStates, NowAdding, StateVectorSize, NumberOfInputs,
             NumberOfStateBits, NumberOfOutputs, SummaryFlag;
  struct State *NewState(), *temp;
  struct StateRecord *CurrentParserRecord;
  extern struct StateRecord *ParserList;
  extern char * ProgramName;
  int NumberOfAllocates;

  StateVectorSize=NumberOfInputs+2*NumberOfStateBits+NumberOfOutputs;
  if(SummaryFlag)
  { fprintf(SummaryFile,
       "%s:  Number of inputs  =%d\n", ProgramName, NumberOfInputs);
    fprintf(SummaryFile,
       "%s:  Number of outputs =%d\n", ProgramName, NumberOfOutputs);
    fprintf(SummaryFile,
       "%s:  Number of states  =%d\n", ProgramName, NumberOfStates);
    fprintf(SummaryFile,
       "%s:  State field size  =%d * 2\n", ProgramName, NumberOfStateBits);
    fprintf(SummaryFile,
       "%s:  Table width       =%d\n", ProgramName, StateVectorSize);
  }

  NumberOfAllocates=0;
  CurrentParserRecord=ParserList;
  while(CurrentParserRecord!=(struct StateRecord *) 0)
  { temp=NewState(CurrentParserRecord->name);
    temp->next=StateList;
    StateList=temp;
    NumberOfAllocates++;
     
    CurrentParserRecord=CurrentParserRecord->next;
  }
  if(NumberOfAllocates!=NumberOfStates)
  {  fprintf(stderr,
     "%s(MakeStateList):  Bad number of allocations\n", ProgramName);
     exit(1);
  }
} /*MakeStateList*/


SameDestination(Left, Right)
/*
 * Return 1 if the Left and Right vectors have the same destination bit
 * values.
 */
   struct StateVectorRow * Left, * Right;
{  int loop, start;
   extern int NumberOfInputs, NumberOfStateBits;
   start=NumberOfInputs+NumberOfStateBits;
   for(loop=0;loop<NumberOfStateBits;loop++)
      if(Left->bits[start+loop]!=Right->bits[start+loop])
      {  /*printf("Bad destination\n");*/
	 return(0);
      }
   return(1);
} /*SameDestination*/


Compatible(Left, Right, FirstColumn, LastColumn)
/*
 * Return the location of the conflict if the Left and Right vectors
 * are compatible for a collapse operation.  Two redundant
 * rows differ by 1<->0, ?<->1, or ?<->0 on inputs.  Otherwise return
 * ILLEGAL, a negative value.
 *
 */
   struct StateVectorRow * Left, * Right;
   int FirstColumn, LastColumn;
{  int loop, conflicts, location;
   extern int NumberOfInputs;
   extern char * ProgramName;

   if(Left==(struct StateVectorRow *)0)
   {  fprintf(stderr, "%s(Compatible):  NULL argument 'Left'\n", ProgramName);
      exit(1);
   }
   if(Right==(struct StateVectorRow *)0)
   {  fprintf(stderr, "%s(Compatible):  NULL argument 'Right'\n", ProgramName);
      exit(1);
   }
   conflicts=0;
   location=ILLEGAL;
   for(loop=FirstColumn;loop<=LastColumn;loop++)
   {  
      switch(Left->bits[loop])
      {  case ZERO    : if(Right->bits[loop]!=ZERO)
	 	        { location=loop;
		          /*printf("  Incompatible at %d\n", loop);*/
	   	          conflicts++;
	                }
	                break;
         case ONE     : if(Right->bits[loop]!=ONE)
	 	        { location=loop;
		          /*printf("  Incompatible at %d\n", loop);*/
	   	          conflicts++;
	 	        }
	 	        break;
         case DONTCARE: if(Right->bits[loop]!=DONTCARE)
		        { location=loop;
		          /*printf("  Incompatible at %d\n", loop);*/
		          conflicts++;
		        }
		        break;
         default      : fprintf(stderr,
			   "Bad value in InputsFit=%d\n", Left->bits[loop]);
		        exit(1);
      }
   }
   if(conflicts<2)
      return(location);
   else
      return(-2);
} /*Compatible*/


DeleteVector(List, ToDelete)
/*
 * Delete the ToDelete vector from List.  It must not be the first item
 * on the list.
 */
   struct StateVectorRow * List, * ToDelete;
{  struct StateVectorRow * temp;

   if(ToDelete==(struct StateVectorRow *) 0)
   {  fprintf(stderr, "DeleteVector:  nil ToDelete\n");
      exit(1);
   }
   temp=List;
   while(temp!=(struct StateVectorRow *) 0)
   {  if(temp->next==(struct StateVectorRow *) 0) /*nil*/
      {  fprintf(stderr, "DeleteVector:  target not on list\n");
	 exit(1);
      }
      if(temp->next==ToDelete)
      {  temp->next=temp->next->next;
	 return;
      }
      temp=temp->next;
   }
} /*DeleteVector*/


Collapse(List, FirstColumn, LastColumn, LastPart)
/*
 * Remove redundant row vectors from the vectors on the list.
 * Two redundant rows have the same destination and differ
 * by 1<->0, ?<->1, or ?<->0 on inputs.  Don't need to worry
 * about state bits or outputs:  they are guaranteed to be
 * the same.  Note:  always collapse by saving the first vector
 * of the two redundant.
 *
 * Return the number of collapsed rows.
 */
   struct StateVectorRow * List;
   int FirstColumn, LastColumn, LastPart;
{  int save, NumberRemoved, Compatible();
   struct StateVectorRow * left, * right;

   /*PrintVectors(List, "loop");*/
   NumberRemoved=0;
   left=List;
   while(left!=(struct StateVectorRow *) 0)
   {  right=left->next;
      while(right!=(struct StateVectorRow *) 0)
      {
         if(SameDestination(left, right)||LastPart)
         {  
            save=Compatible(left, right, FirstColumn, LastColumn);
	  /*printf("Save %d,Fst %d,Lst %d\n", save, FirstColumn, LastColumn);*/
	    if(save!= -2)
	    {  /*printf("\nCollapsing these two vectors:\n");
	         PrintVector(left, "foo");
	         PrintVector(right, "foo");
	        */
		if(save!=ILLEGAL)
	           left->bits[save]=DONTCARE;
	        DeleteVector(List, right);
	        NumberRemoved++;
	    }
	 }
	 right=right->next;
      }
      left=left->next;
   }
   if(NumberRemoved>0)
   {  /*printf("Here is the collapsed list:\n");
        PrintVectors(List, "foo");
        printf("\n\n");
       */
      return(NumberRemoved+Collapse(List, FirstColumn, LastColumn, LastPart));
   }
   else return(NumberRemoved);
} /*Collapse*/
