/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Vectors.c	40.1      peg      6/29/88"               */

/*
 * Vectors.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)Vectors.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint


#include "LookupDefs.i"
#include "Externals.i"

SetStateBits(VectorRow, StateName, InOrOut)
/*************************************************************************
 * Set either the current state bits or the next state bits in VectorRow->
 * to the state number of the given state.
 *
 * The format of the state vector is:
 *   Current State Bits, Inputs, Next State Bits, Outputs
 *************************************************************************
 */
   struct StateVectorRow *VectorRow;
   char * StateName;
   int InOrOut;
{  int MSBIndex, i, temp;
   extern int NumberOfStateBits, NumberOfInputs;
   extern char * ProgramName;
   unsigned mask;
   /*Decide which set of bits to set-up*/
   if(InOrOut==INPUTS)
      MSBIndex=NumberOfInputs;
   else
   if(InOrOut==OUTPUTS)
      MSBIndex=2*NumberOfStateBits+NumberOfInputs-1;
   else
   {  fprintf(stderr, 
      "%s(SetStateBits):  Bad value for InOrOutSetStateBits.\n", ProgramName);
      exit(1);
   }

   temp=NumberFromName(StateName);
   if(temp== -1)
   {  fprintf(stderr, "%s(SetStateBits):  State %s not found in lookup\n",
         ProgramName, StateName);
      abort();
   }
   mask= 1 << (NumberOfStateBits-1);
   for(i=0; i<NumberOfStateBits; i++)
   {  if( (temp&(mask>>i)) !=0 )
      {  if(InOrOut==INPUTS)
	    VectorRow->bits[i+MSBIndex]=ONE;
	 else
	    VectorRow->bits[MSBIndex-i]=ONE;
      }
      else
      {	 if(InOrOut==INPUTS)
            VectorRow->bits[i+MSBIndex]=ZERO;
	 else
            VectorRow->bits[MSBIndex-i]=ZERO;
      }
   }
} /*SetStateBits*/


struct StateVectorRow*
NewStateVector()
/*************************************************************
 * Return a pointer to a default-initialized state vector row.
 *************************************************************
 */
{  struct StateVectorRow *temp;
   extern int StateVectorSize;
   extern char * ProgramName;
   int i;
   char *calloc();

   temp=(struct StateVectorRow *) calloc(1, sizeof(struct StateVectorRow));
   if(temp==0)
   {  fprintf(stderr, "%s(NewStateVector):  calloc failed\n", ProgramName);
      exit(1);
   };

   temp->bits=(char *) calloc(StateVectorSize+1, sizeof(char));
   if(temp->bits==0)
   {  fprintf(stderr, "%s(NewStateVector):  calloc failed\n", ProgramName);
      exit(2);
   };

   for(i=0;i<StateVectorSize;i++)
      temp->bits[i]=DONTCARE;
   temp->bits[StateVectorSize]='\0';
   temp->next=NULL;
   return(temp);
} /*NewStateVector*/


struct StateVectorRow *
CopyVector(ToCopy)
/****************************************************
 * Return a pointer to a copy of the given row vector.
 ****************************************************
 */
   struct StateVectorRow * ToCopy;
{  struct StateVectorRow * NewStateVector(), *temp;
   extern int StateVectorSize;
   extern char *ProgramName;
   int i;

   if(ToCopy==NULL)
   {  fprintf(stderr, "%s(CopyVector):  NULL pointer passed.\n", ProgramName);
      exit(1);
   }
   temp=NewStateVector();
   for(i=0;i<StateVectorSize;i++)
      temp->bits[i]=ToCopy->bits[i];
   temp->next=ToCopy->next;
   return(temp);
} /*CopyVector*/
