/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)acts.c	40.1      peg      6/29/88"               */

/*
 * acts.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)acts.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint


#include "LookupDefs.i"
#include "ParserDefs.i"

AddListItems(ItemType, List)
/*
 * Add the list of identifiers to the symbol table, giving them
 * the proper symbol type.  Since the list is organized backwards,
 * restore the items to the proper order by doing them recursively
 * from the end.
 */
   int             ItemType;
   struct IdList * List;
{  extern char * ProgramName;
   if(List==(struct IdList *)0)
      return;
   else
   {  AddListItems(ItemType, List->next);
    /*printf("AddListItems adding '%s' of type %d\n", List->name, ItemType);*/
      if(strcmp(List->name, "LOOP")==0)
      { fprintf(stderr, "%s(AddListItems):", ProgramName);
	error("Keyword LOOP redeclared");
	exit(1);
      }
      switch(ItemType)
      { case INPUTS : AddInput(List->name); break;
	case OUTPUTS: AddOutput(List->name); break;
	case STATE  : AddState(List->name); break;
	default     : { fprintf(stderr, "Error in AddListItems\n");
			fprintf(stderr, "  Bad type %d for %s\n",
				ItemType, List->name);
			exit(1);
			break;
		      }
      }
   }
} /*AddListItems*/


IdListLength(List)
/*
 * Return the number of identifiers on the list.
 */
   struct IdList * List;
{  struct IdList * temp;
   int result;
   temp=List;
   result=0;
   while(temp!=(struct IdList *)0)
   {  result++;
      temp=temp->next;
   }
   return(result);
} /*IdListLength*/


struct CaseItem *
NewCaseLine(BitList, NextState)
/*
 * Allocate a new case record.
 */
   struct IdList * BitList;
   char          * NextState;
{  struct CaseItem * result;
   struct StateVectorRow * NewStateVector();
   char * calloc();
   /*printf("Allocating a new case line record with next %s\n", NextState);*/
   result=(struct CaseItem *) calloc(1, sizeof(struct CaseItem));
   result->bits=BitList;
   result->StateName=NextState;
   result->caseUsed=0;
   result->next=(struct CaseItem *) 0;

   /*printf("   NewCaseLine returns this...");
    *PrintCaseList(result);
    */
   return(result);
} /*NewCaseLine*/


struct IdList *
NewIdRec(IdName)
/*
 * Return a pointer to a new id record with the given name.
 */
   char * IdName; 
{  struct IdList * result;
   char * calloc();
   result=(struct IdList *) calloc(1, sizeof(struct IdList));
   result->name=IdName;
   result->next=(struct IdList *) 0;
   return(result);
} /*NewIdRec*/


struct StateRecord *
NewStateRecord(InputList, CaseList, Default)
/*
 * Allocate a new state record with no name, no assertions,
 * 	Branch Selector <- InputList
 *      Branch List     <- CaseList
 *      Next	  	<- nil
 * The first item on the case list is the default action to be
 * performed if none of the case lines match.
 */
   struct IdList   * InputList;
   struct CaseItem * CaseList;
   char            * Default;
{  struct StateRecord * result;
   struct CaseItem * NewCaseLine();
   char * calloc();

   result=(struct StateRecord *) calloc(1, sizeof(struct StateRecord));
   result->name    =(char *) 0;
   result->assert  =(struct IdList *) 0;
   result->selector=InputList;
   result->cases   =NewCaseLine((struct IdList *) 0, Default);

   /*printf("***Should be a single item...\n");
    *PrintCaseList(result->cases);
    *printf("***Should be a list of items...\n");
    *PrintCaseList(CaseList);
    */
   result->cases->next=CaseList;
   /*printf("***Should be the list of all itmes...\n");
    *PrintCaseList(result->cases);
    */
   result->next    =(struct StateRecord *) 0;
   return(result);
} /*NewStateRecord*/


PrintCaseList(List)
/*
 * Print the data on the linked list of case lines.
 */
   struct CaseItem * List;
{  struct CaseItem * temp;
   printf("\n   Print Case List\n");
   temp=List;
   while(temp!=(struct CaseItem *)0)
   { printf("...printing...");
     PrintIdList(temp->bits, "=>");
     printf("   %s\n", temp->StateName);
     temp=temp->next;
   }
} /*PrintCaseList*/


PrintStateList(List)
/*
 * Print the data on the linked list of states.
 */
   struct StateRecord * List;
{  struct StateRecord * temp;
   printf("DUMP of State List\n");
   temp=List;
   while(temp!=(struct StateRecord *)0)
   { printf("State '%s':  ", temp->name);
     printf("   Assert ");
     PrintIdList(temp->assert, ", ");
     PrintCaseList(temp->cases);
     temp=temp->next;
   }
} /*PrintStateList*/


PrintIdList(List, Spacer)
/*
 * Print the names on the linked list of identifiers.
 */
   struct IdList * List;
   char          * Spacer;
{  struct IdList * temp;
   temp=List;
   while(temp!=(struct IdList *) 0)
   {  printf("   %s%s", temp->name, Spacer);
      temp=temp->next;
   }
} /*PrintIdList*/


PrintType(TypeCode)
   int TypeCode;
{
	switch(TypeCode)
	{  case INPUTS :  fprintf(stderr, "'input' "); break;
	   case OUTPUTS:  fprintf(stderr, "'output' "); break;
	   case STATE  :  fprintf(stderr, "'state' "); break;
	   default     :  fprintf(stderr, "<undeclared> "); break;
	}
} /*PrintType*/


TypesAreOkay(SymbolList, Type)
/*
 * Check to see if all of the symbols on the symbol list have the given
 * integer type code.  If not, then return 0, else return 1.
 *
 */
   struct IdList * SymbolList;
   int             Type;
{  extern char * ProgramName;
   struct IdList * temp;
   int TypeCode;
   temp=SymbolList;
   while(temp!=(struct IdList *) 0)
   { TypeCode=TypeFromName(temp->name);
     if(TypeCode!=Type)
     {  fprintf(stderr,
	  "%s(TypesAreOkay):  Identifier %s of type ", ProgramName, temp->name);
	PrintType(TypeCode);
	fprintf(stderr, "used where type ");
	PrintType(Type);
	error("was expected\nError");
	return(0);
     }
     else
	temp=temp->next;
   }
   return(1);
} /*TypesAreOkay*/

