/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)gen.c	40.1      peg      6/29/88"               */

/*
 * gen.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)gen.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint


#include <stdio.h>
#include "ParserDefs.i"
#include "LookupDefs.i"

char * StateBaseName;
int    StateOffset;


FixOutputCounters()
/*
 *  Traverse the symbol table list, adding Offset to each OUTPUT entry's
 *  number.
 */
{  extern int NumberOfInputs, NumberOfStateBits;
   extern struct DataItem * SymbolList;

   struct DataItem * temp;
   int BitsPerState(), Offset;

   NumberOfStateBits=BitsPerState();
   Offset=(2*NumberOfStateBits)+NumberOfInputs;
   temp=SymbolList;
   while(temp!=(struct DataItem *)0)
   {  if(temp->type==OUTPUTS)
	 temp->number=temp->number+Offset;
      temp=temp->next;
   }
} /*FixOutputCounters*/


BuildTable(List)
/*
 * Traverse the list of state records to build up the state vectors
 * used by the equations generator.
 */
   struct StateRecord * List;
{  struct StateRecord * CurrentState;
   struct State       * StateFromName(), * EquationData;
   char               * DefaultNextState;
   
   DefaultNextState=(char *) 0;
   CurrentState=List;
   while(CurrentState!=(struct StateRecord *) 0)
   {
      /*printf("   Working on state %s\n", CurrentState->name);*/
      if(CurrentState->selector==(struct IdList *) 0)
      {/*
	* Indicates a GOTO
	*/
	  if(CurrentState->cases->StateName==(char *) 0)
	  {/*
	    * No explicit next state is specified.  Use default.
	    */
	      /*printf("   State %s is default GOTO\n", CurrentState->name);*/
	      if(DefaultNextState==(char *) 0)
	      {  fprintf(stderr, "State %s:  no NEXT state defined.\n", CurrentState->name);
	         exit(1);
	      }
	      EquationData=StateFromName(CurrentState->name);
	      SetStateBits(EquationData->vectors, DefaultNextState, OUTPUTS);
	  }
	  else
          {/*
	    * Indicates an explicit GOTO
	    */
	      /*printf("   State %s is explicit GOTO\n", CurrentState->name);*/
	      EquationData=StateFromName(CurrentState->name);
	      if(strcmp(CurrentState->cases->StateName, "LOOP")!=0)
	         SetStateBits(EquationData->vectors,
		 CurrentState->cases->StateName, OUTPUTS);
	      else
		 SetStateBits(EquationData->vectors,
		 CurrentState->name, OUTPUTS);
          }
      }
      else
      {/*
	* Either IF-THEN-ELSE or IF-THEN or CASE.  Handle in usual way.
	*/
	  /*printf("   IF-THEN-ELSE, IF-THEN, or CASE\n");*/
	  if(CurrentState->cases->StateName==(char *) 0) /*Undefined next*/
	  {  
	     /*printf("   State %s is CASE with no specific default\n",
	      *	         CurrentState->name);
	      */
	     CurrentState->cases->StateName=DefaultNextState;
	  }
	  ApplyCases(CurrentState);
      }
      DefaultNextState=CurrentState->name;
      CurrentState=CurrentState->next;
   }
} /*BuildTable*/


RegisterStateNames(List)
/*
 * Traverse the list of state records, starting with the last one.
 * Call AddState with each record's state name.
 */
   struct StateRecord * List;
{  if(List==(struct StateRecord *) 0)
      return;
   else
   {  RegisterStateNames(List->next);
      AddState(List->name);
   }
} /*RegisterStateNames*/


AssertAllSignals(List)
/*
 * Traverse the list of state records.  Call AssertSignal
 * with each identifier on each record's 'assert' list.
 */
   struct StateRecord * List;
{  struct StateRecord * TempState;
   struct IdList      * TempName;
   TempState=List;
   while(TempState!=(struct StateRecord *) 0) /*Traverse list of states*/
   {  TempName=TempState->assert;
      while(TempName!=(struct IdList *) 0) /*Traverse list of signals*/
      {  AssertSignal(TempState->name, TempName->name, ONE);
	 /*printf("      Assert %s in state %s\n",
	   TempName->name, TempState->name);
	  */
	 TempName=TempName->next;
      }
      TempState=TempState->next;
   }
} /*AssertAllSignals*/


char *
RelativeStateName()
/*
 * Generate a character string of the form "<Base>+<Offset>"
 * Example:  Reset+3
 */
{  char * result, * temp, *strcat(), *strcpy(), *calloc();
   if(strlen(StateBaseName)==0)
   {  fprintf(stderr, "Nil base in RelativeStateName\n");
      exit(1);
   }
   if((StateOffset<1)||(StateOffset>32000))
   {  fprintf(stderr, "Bad offset %d in RelativeStateName\n", StateOffset);
      exit(1);
   }
   result=(char *) calloc(strlen(StateBaseName)+20, sizeof(char));
   temp  =(char *) calloc(20, sizeof(char));
   strcpy(result, StateBaseName);
   sprintf(temp, "+%01d\0", StateOffset);
   result=strcat(result, temp);
   return(result);
} /*RelativeStateName*/



